@@ -13,10 +13,13 @@ int debug(char* fmt, ...)
 #ifdef DEBUG
  va_list ap;
  int err;
+ if (!fmt) return -1;
  va_start(ap,fmt);
  printf("debug: ");
  err = vprintf(fmt,ap);
  va_end(ap);
+ printf("\n");
+ fflush(stdout);
  return err;
 #endif
 #ifndef DEBUG
@@ -28,11 +31,13 @@ int debug(char* fmt, ...)
 int fatal(char* fmt, ...)
 {
  va_list ap;
+ if (!fmt) return -1;
  va_start(ap,fmt);
  printf("\n\nPANIC\n\t%c%cfatal: ",7,7);
  vprintf(fmt,ap);
  printf("\nApplication Terminated.%c%c\n\n", 7,7);
  va_end(ap);
+ fflush(stdout);
  exit(1);
 }
 
@@ -41,6 +46,7 @@ void downcase(char* ptr)
 {
  int i;
  int s;
+ debug("%s:%d",__FILE__,__LINE__);
  if (!ptr) return;
  s = strlen(ptr);
  for (i=0;i<s;i++) if (ptr[i]>='A' && ptr[i]<='Z') ptr[i]+=0x20;
@@ -51,10 +57,24 @@ int error(char* fmt, ...)
 {
  va_list ap;
  int err;
+ if (!fmt) return -1;
  va_start(ap,fmt);
  printf("error: ");
  err = vprintf(fmt,ap);
  va_end(ap);
+ fflush(stdout);
+ return err;
+}
+
+int warn(char* fmt, ...)
+{
+ va_list ap;
+ int err;
+ if (!fmt) return -1;
+ va_start(ap,fmt);
+ printf("warning: ");
+ err = vprintf(fmt,ap);
+ va_end(ap);
  return err;
 }
 
@@ -63,6 +83,8 @@ int system_printf(char* fmt, ...)
 {
  va_list ap;
  char syscmd[MAX_SYSTEM_CMD];
+ debug("%s:%d",__FILE__,__LINE__);
+ if (!fmt) return -1;
  va_start(ap,fmt);
  vsprintf(syscmd,fmt,ap);
  va_end(ap);
@@ -73,11 +95,12 @@ int system_printf(char* fmt, ...)
 void print_results(char*** res, int a, int b)
 {
  int i,j;
+ debug("%s:%d",__FILE__,__LINE__);
  ln();
  if (!res) return;
  for (i=0;i<a;i++)
    {
-    for (j=0;j<b;j++) printf("%-16s ", res[i][j]);
+    if (res[i]) for (j=0;j<b;j++) if (res[i][j]) printf("%-16s ", res[i][j]);
     ln();
    }
  ln();
