@@ -10,17 +10,19 @@
 #define __MORGOTHDBMA_GTKTEST__
 #include "gtkincl.h"
 #include "query.h"
+#include "consoleout.h"
 #define ANSWER_YES 1
 #define ANSWER_NO  0
-#define MAX_ENTRY_LENGTH 512
-#define MAX_MSG_LENGTH   512
+#define MAX_ENTRY_LENGTH 1024
+#define MAX_MSG_LENGTH   4608
+#define MASK_LENGTH        32
 int gtk_dialog_yes_no(const char*);
 int gtk_dialog_with_2_buttons(const char*, char*, char*);
 int gtk_dialog_yes_no_printf(const char*, ...);
 char* gtk_dialog_gettext(const char*);
 char* gtk_dialog_gettext_printf(const char*, ...);
-char* gtk_dialog_getfile_printf(const char*, ...);
-char* gtk_dialog_getfile(const char*);
+char* gtk_dialog_getfile_printf(const char*, const char*, ...);
+char* gtk_dialog_getfile(const char*, const char*);
 void gtk_dialog_mbox(const char*, const char*);
 void gtk_dialog_printf_with_title(const char*, const char*, ...);
 void gtk_dialog_printf_big_with_title(const char*, const char*, ...);
