@@ -10,32 +10,37 @@
  ! BUT I DON'T KNOW THEIR PERSONALITY SO SORRY !
  ! I CANNOT ADD THEM TO AUTORS, BUT THANKS FOR SUPPORT ME !
  FILE: main.c */
-
 #include "main.h"
 #include "pgxexplorer.h"
 
 void disconnect()
 {
+ debug("%s:%d",__FILE__,__LINE__);
  if (connected) disconnect_db(&connection);
  connected=0;
  connection=0;
 }
 
-void __dummy( GtkWidget *w,gpointer   data )
+
+void __dummy( GtkWidget *w,gpointer data)
 {
-  error("Not yet implemented.\n");
+ debug("%s:%d",__FILE__,__LINE__);
+ error("Not yet implemented.\n");
 }
 
 
-void close_results( GtkWidget *w,gpointer   data)
+void close_results( GtkWidget *w,gpointer data)
 {
+ debug("%s:%d",__FILE__,__LINE__);
  gtk_main_quit();
+ if (!data) return;
  gtk_widget_destroy((GtkWidget*)(data));
 }
 
 
-void qexec_quit( GtkWidget *w,gpointer   data)
+void qexec_quit( GtkWidget *w,gpointer data)
 {
+ debug("%s:%d",__FILE__,__LINE__);
  gtk_main_quit();
  gtk_widget_destroy(query_exec);
  query_exec = NULL;
@@ -44,6 +49,7 @@ void qexec_quit( GtkWidget *w,gpointer   data)
 
 int menu_quit(GtkWidget *w, gpointer data)
 {
+ debug("%s:%d",__FILE__,__LINE__);
   if (gtk_dialog_yes_no("Are you sure?"))
    {
     disconnect();
@@ -63,9 +69,69 @@ int menu_quit(GtkWidget *w, gpointer data)
 
 void about_box( GtkWidget *w,gpointer   data)
 {
+ debug("%s:%d",__FILE__,__LINE__);
  about_window_new();
 }
 
+void copy_database(GtkWidget* w, gpointer data)
+{
+ int emerg;
+ char *dbname, dbs[MAXPATH];
+ char errstr[MAX_QUERY_LENGTH];
+ char db2[MAXPATH];
+ char winname[STDSTRING];
+ char*** ptr;
+ int row,col;
+ debug("%s:%d",__FILE__,__LINE__);
+ emerg = !connected;
+ if (!connected && !dummy_connect(1,TEMPLATE))
+    {
+     gtk_dialog_printf("Cannot connect to template DB\nCannot get DB list.");
+     return ;
+    }
+ dbname = gtk_dialog_getdbs(connection, "Which DB copy?");
+ if (emerg) { dummy_connect(0,TEMPLATE); connected=0; }
+ strcpy(dbs, dbname);
+ dbname = gtk_dialog_gettext_printf("Copy %s to:", dbs);
+ strcpy(db2, dbname);
+ if (!strcmp(db2,""))
+      {
+	gtk_dialog_printf("Cannot save %s to empty name.\n", dbs);
+        return;
+      }
+ emerg = !connected;
+ if (!connected && !dummy_connect(1,TEMPLATE))
+   {
+    gtk_dialog_printf("Cannot connect to template DB\nResult: Cannot create new DB");
+    return ;
+   }
+ execute_printf(connection, &ptr, &row, &col, errstr, 0, "CREATE DATABASE %s", db2);
+ if (emerg) { dummy_connect(0,TEMPLATE); connected = 0; }
+ if (row<0 || col<0)
+   {
+    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+    return;
+   }
+ free_p3c(&ptr, row, col);
+ system_printf("pg_dump -i %s | psql %s", dbs,db2);
+ if (!emerg) return;
+ connection = gtk_connect_db(db2, errstr);
+ if (!connection)
+     {
+      fatal_nodb(db2,errstr,0);
+      return ;
+     }
+ else
+     {
+      connected=1;
+      currentbase = (char*)malloc(strlen(db2)+2);
+      strcpy(currentbase, db2);
+      sprintf(winname, "PgXexplorer: %s", currentbase);
+      gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
+     }
+}
+
+
 void create_DB_from_file(GtkWidget* w, gpointer data)
 {
  char *filen,*sfilen,*dbn;
@@ -75,6 +141,7 @@ void create_DB_from_file(GtkWidget* w, gpointer data)
  char*** ptr;
  int l,row,col,emerg;
  char winname[STDSTRING];
+ debug("%s:%d",__FILE__,__LINE__);
  dbn = gtk_dialog_gettext_printf("Need DB name to fill from a SQL file:");
  if (!strcmp(dbn,""))
     {
@@ -82,10 +149,10 @@ void create_DB_from_file(GtkWidget* w, gpointer data)
      return;
     }
  strcpy(dbname,dbn);
- filen = gtk_dialog_getfile("Select SQL dump file (*.SQL)");
+ filen = gtk_dialog_getfile(".SQL", "Select SQL dump file (*.SQL)");
  if (!strcmp(filen,""))
    {
-    gtk_dialog_printf("Cannot load DB from an empty name\n");
+    gtk_dialog_printf("Cannot load DB from non-SQL file\n");
     return;
    }
  l = strlen(filen);
@@ -107,12 +174,12 @@ void create_DB_from_file(GtkWidget* w, gpointer data)
    }
  fclose(f);
  emerg = !connected;
- if (!connected && !dummy_connect(1,TEMPLATE)) 
+ if (!connected && !dummy_connect(1,TEMPLATE))
    {
     gtk_dialog_printf("Cannot connect to template DB\nResult: Cannot create new DB");
     return ;
    }
- execute_printf(connection, &ptr, &row, &col, errstr, "CREATE DATABASE %s", dbname);
+ execute_printf(connection, &ptr, &row, &col, errstr, 0, "CREATE DATABASE %s", dbname);
  if (emerg) { dummy_connect(0,TEMPLATE); connected = 0; }
  if (row<0 || col<0)
    {
@@ -158,19 +225,20 @@ void save_database(GtkWidget* w, gpointer data)
  char* dbname;
  char dbs[STDSTRING];
  int emerg;
+ debug("%s:%d",__FILE__,__LINE__);
  if (!connected) a=0;
- else 
+ else
  a = gtk_dialog_with_2_buttons("Do You want to save Current database or Other database?", "Current", "Other");
  if (a)
    {
     /*filename = gtk_dialog_gettext_printf("Enter filename, %s will be saved to:\n(extension .SQL will be added automatically)", currentbase);*/
-    filename = gtk_dialog_getfile_printf("Enter filename, %s will be saved to", currentbase);
+    filename = gtk_dialog_getfile_printf(".SQL", "Enter filename, %s will be saved to", currentbase);
     if (!strcmp(filename,""))
       {
-	gtk_dialog_printf("Cannot save %s to empty name.\n", currentbase);
+	gtk_dialog_printf("Cannot save %s to non-SQL file", currentbase);
         return;
       }
-    system_printf("pg_dump -i '%s' > '%s.SQL'", currentbase, filename);
+    system_printf("pg_dump -i '%s' > '%s'", currentbase, filename);
    }
  else
  {
@@ -185,13 +253,13 @@ void save_database(GtkWidget* w, gpointer data)
   strcpy(dbs, dbname);  /* HEADERS WILL USE OLD POINTER IN DIALOGS, SO MUST COPY IT */
   			/* NOW VARIABLE dbs IS A COPY, AND dbname CAN (NOT MUST) */
   			/* BE CHANGED BY DIALOGS */
-  filename = gtk_dialog_getfile_printf("Enter filename, %s will be saved to", dbs);
+  filename = gtk_dialog_getfile_printf(".SQL", "Enter filename, %s will be saved to",dbs);
   if (!strcmp(filename,""))
       {
-	gtk_dialog_printf("Cannot save %s to empty name.\n", dbs);
+	gtk_dialog_printf("Cannot save %s to non-SQL file\n", dbs);
         return;
       }
-  system_printf("pg_dump -i '%s' > '%s.SQL'", dbs, filename);
+  system_printf("pg_dump -i '%s' > '%s'", dbs, filename);
  }
 }
 
@@ -199,6 +267,7 @@ char* get_env(char*);
 
 char *dbenv()
 {
+ debug("%s:%d",__FILE__,__LINE__);
  sprintf(tmpstr,"PGDATA=\"%s\"\nPGPORT=\"%s\"\nPGHOST=\"%s\"", get_env("PGDATA"), get_env("PGPORT"), get_env("PGHOST"));
  return tmpstr;
 }
@@ -206,11 +275,12 @@ char *dbenv()
 
 void fatal_nodb(char* dbname, char* errstr, int killed)
 {
+ debug("%s:%d",__FILE__,__LINE__);
  if (killed)
  gtk_dialog_printf_big_with_title("PgXexplorer Fatal Error",
  	"Cannot connect to DB: %s\nDBMS returned: \n%s\nSystem will now exit.\nENV=\n%s",
   	dbname,errstr,dbenv());
- else 
+ else
  gtk_dialog_printf_big_with_title("PgXexplorer Error",
  	"Cannot connect to DB: %s\nDBMS returned: \n%s\nENV=\n%s",
   	dbname,errstr,dbenv());
@@ -224,6 +294,7 @@ void reload_bases(char* db, int ask)
  char errstr[MAX_QUERY_LENGTH];
  char winname[STDSTRING];
  char dbname[STDSTRING];
+ debug("%s:%d",__FILE__,__LINE__);
  strcpy(dbname,db);
  if (ask) a = gtk_dialog_yes_no_printf("Switch to database \"%s\" now?", db);
  else a=1;
@@ -262,8 +333,9 @@ void load_database(GtkWidget* w, gpointer data)
  char winname[STDSTRING];
  char errstr[MAX_QUERY_LENGTH];
  int emerg;
+ debug("%s:%d",__FILE__,__LINE__);
  emerg = !connected;
- if (!connected && !dummy_connect(1,TEMPLATE)) 
+ if (!connected && !dummy_connect(1,TEMPLATE))
    {
     gtk_dialog_printf("Cannot connect to template DB\nServer is dead?");
     return;
@@ -311,6 +383,7 @@ void drop_database(GtkWidget* w, gpointer data)
  char errstr[MAX_QUERY_LENGTH];
  PGresult *result;
  int emerg;
+ debug("%s:%d",__FILE__,__LINE__);
  emerg = !connection;
  if (!connected && !dummy_connect(1,TEMPLATE))
    {
@@ -331,18 +404,18 @@ void drop_database(GtkWidget* w, gpointer data)
     }
  if (strstr(dbname, "template"))
    {
-    if (emerg) 
-      { 
-       dummy_connect(0,TEMPLATE); 
-       connected = 0; 
+    if (emerg)
+      {
+       dummy_connect(0,TEMPLATE);
+       connected = 0;
        gtk_dialog_printf("Cannot drop template\nwhile it is beeing used\nas emergency DB!");
        return ;
       }
     if (!gtk_dialog_yes_no_printf(
-    "Are You sure want to drop\ntemplate DB: %s\nThis can prevent PgXexplorer working properly",dbname)) 
+    "Are You sure want to drop\ntemplate DB: %s\nThis can prevent PgXexplorer working properly",dbname))
 	    return ;
    }
- execute_printf_query(connection, &result, errstr, "DROP DATABASE %s", dbname);
+ execute_printf_query(connection, &result, errstr, 0, "DROP DATABASE %s", dbname);
  if (emerg) { dummy_connect(0,TEMPLATE); connected = 0; }
  if (!result)
    {
@@ -360,6 +433,7 @@ void create_database(GtkWidget* w, gpointer data)
  int a;
  int emerg;
  PGresult *result;
+ debug("%s:%d",__FILE__,__LINE__);
  emerg=!connected;
  dbname = gtk_dialog_gettext("Enter database name: ");
  if (!strcmp(dbname,""))
@@ -368,12 +442,12 @@ void create_database(GtkWidget* w, gpointer data)
      return;
     }
  result = NULL;
- if (!connected && !dummy_connect(1,TEMPLATE)) 
+ if (!connected && !dummy_connect(1,TEMPLATE))
    {
     gtk_dialog_printf("Cannot connect to template DB\nResult: Cannot create new DB");
     return ;
    }
- execute_printf_query(connection, &result, errstr, "CREATE DATABASE %s", dbname);
+ execute_printf_query(connection, &result, errstr, 0, "CREATE DATABASE %s", dbname);
  if (emerg) { dummy_connect(0,TEMPLATE); connected = 0; }
  if (!result)
    {
@@ -412,6 +486,7 @@ void create_database(GtkWidget* w, gpointer data)
 
 void exec_test_prog(GtkWidget* w, gpointer data)
 {
+ debug("%s:%d",__FILE__,__LINE__);
  system("./pgxtest");
 }
 
@@ -421,6 +496,7 @@ void gtk_table_out(char* title, char*** str_data, int a, int b)
  GtkWidget* window, *table, *button, *scrolled_window;
  GtkWidget*** entry;
  int i,j;
+ debug("%s:%d",__FILE__,__LINE__);
  if (!str_data) { gtk_dialog_printf("Error, table of results to display is equal NULL"); return; }
  entry = (GtkWidget***)malloc(a*sizeof(GtkWidget**));
  for (i=0;i<a;i++) entry[i] = (GtkWidget**)malloc(b*sizeof(GtkWidget*));
@@ -483,7 +559,9 @@ void test_queries()
 
 void clear_txt_ctrl(GtkWidget* w)
 {
- int l = gtk_text_get_length(GTK_TEXT(w));
+ int l;
+ debug("%s:%d",__FILE__,__LINE__);
+ l = gtk_text_get_length(GTK_TEXT(w));
  gtk_text_set_point(GTK_TEXT(w), 0);
  gtk_text_forward_delete(GTK_TEXT(w), l);
 }
@@ -493,17 +571,18 @@ void load_proc(GtkWidget* w, gpointer txt)
 {
  char* filen;
  FILE* f;
- int l;
+ int l,max;
  char* str;
+ debug("%s:%d",__FILE__,__LINE__);
  if (!connected)
    {
     gtk_dialog_printf("Connect to DB firest!");
     return;
    }
- filen = gtk_dialog_getfile("Select query file (*.QUE)");
+ filen = gtk_dialog_getfile(".QUE", "Select query file (*.QUE)");
  if (!strcmp(filen,""))
    {
-    gtk_dialog_printf("Cannot load query from an empty name\n");
+    gtk_dialog_printf("Cannot load query from non-QUE file\n");
     return;
    }
  f = fopen(filen,"r");
@@ -516,9 +595,15 @@ void load_proc(GtkWidget* w, gpointer txt)
  fseek(f,0,SEEK_END);
  l=ftell(f);
  fseek(f,0,SEEK_SET);
+ if (l>=MAX_QUERY_LENGTH-100)
+   {
+    gtk_dialog_printf("File is too long!\nTruncating...");
+    l = MAX_QUERY_LENGTH-100;
+   }
  str = malloc(l+2);
+ max=l;
  l=0;
- while ((str[l++]=fgetc(f))!=EOF) ;
+ while (l<=max && (str[l++]=fgetc(f))!=EOF) ;
  str[--l]=0;
  gtk_text_insert(GTK_TEXT(txt), NULL,NULL,NULL,str,l);
  fclose(f);
@@ -532,6 +617,7 @@ void save_proc(GtkWidget* w, gpointer txt)
  char filename[MAXPATH];
  FILE* f;
  int l,i;
+ debug("%s:%d",__FILE__,__LINE__);
  l = gtk_text_get_length(GTK_TEXT(txt));
  if (l<3)
     {
@@ -541,15 +627,14 @@ void save_proc(GtkWidget* w, gpointer txt)
  t = malloc(l+2);
  for (i=0;i<l;i++) t[i] = GTK_TEXT_INDEX(GTK_TEXT(txt), i);
  t[i] = 0;
- filen = gtk_dialog_getfile("Enter filename to save query");
+ filen = gtk_dialog_getfile(".QUE", "Enter filename to save query");
  if (!strcmp(filen,""))
    {
-    gtk_dialog_printf("Cannot save query to empty name.\n");
+    gtk_dialog_printf("Cannot save query to non-QUE file\n");
     free(t);
     return;
    }
  strcpy(filename,filen);
- strcat(filename,".QUE");
  f = fopen(filename,"r");
  if (f)
    {
@@ -579,14 +664,20 @@ void execute_proc(GtkWidget* w, gpointer txt)
  char* t;
  int l,i,a,b;
  PGresult *result;
- char errstr[MAX_QUERY_LENGTH];
+ char errstr[MAX_QUERY_LENGTH+0x100];
  char*** out;
+ debug("%s:%d",__FILE__,__LINE__);
  if (!connected)
    {
     gtk_dialog_printf("Connect to DB first!");
     return;
    }
  l = gtk_text_get_length(GTK_TEXT(txt));
+ if (l>=MAX_QUERY_LENGTH-100)
+   {
+    gtk_dialog_printf("Query too long!\n");
+    return ;
+   }
  if (l<3)
     {
      gtk_dialog_printf("Will not execute empty query\nTry once again.");
@@ -603,7 +694,7 @@ void execute_proc(GtkWidget* w, gpointer txt)
     gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
     return;
    }
- out = pgres2pc3(result, &a, &b);
+ out = pgres2pc3(result, &a, &b, 1);
  clear_result(&result);
  if (a>0 && b>0) gtk_table_out("Results of Query:", out, a, b);
  else gtk_dialog_printf("Query was OK, but didn't returned any result\nThis probably was INSERT OR CREATE...");
@@ -613,6 +704,7 @@ void execute_proc(GtkWidget* w, gpointer txt)
 
 void update_entry(GtkWidget* w, gpointer d)
 {
+ debug("%s:%d",__FILE__,__LINE__);
  strcpy((char*)d,gtk_entry_get_text(GTK_ENTRY(w)));
 }
 
@@ -620,6 +712,7 @@ void update_entry(GtkWidget* w, gpointer d)
 void apply_proc(GtkWidget* w, gpointer d)
 {
  char** ptr;
+ debug("%s:%d",__FILE__,__LINE__);
  ptr = (char**)d;
  if (setenv("PGPORT",ptr[0],1)==-1) gtk_dialog_printf("Cannot set environment variable: PGPORT");
  if (setenv("PGDATA",ptr[1],1)==-1) gtk_dialog_printf("Cannot set environment variable: PGDATA");
@@ -631,6 +724,7 @@ void apply_proc(GtkWidget* w, gpointer d)
 int connect_proc(GtkWidget* w, gpointer d)
 {
  char** ptr;
+ debug("%s:%d",__FILE__,__LINE__);
  ptr = (char**)d;
  if (setenv("PGPORT",ptr[0],1)==-1) gtk_dialog_printf("Cannot set environment variable: PGPORT");
  if (setenv("PGDATA",ptr[1],1)==-1) gtk_dialog_printf("Cannot set environment variable: PGDATA");
@@ -643,6 +737,7 @@ int connect_proc(GtkWidget* w, gpointer d)
 
 int delete_proc(GtkWidget* w, gpointer d)
 {
+ debug("%s:%d",__FILE__,__LINE__);
  gtk_main_quit();
  /*gtk_widget_destroy((GtkWidget*)d);*/
  return TRUE;
@@ -652,6 +747,7 @@ int delete_proc(GtkWidget* w, gpointer d)
 char* get_env(char* str)
 {
  char* ret;
+ debug("%s:%d",__FILE__,__LINE__);
  ret = getenv(str);
  if (ret) return ret;
  else return "";
@@ -663,6 +759,7 @@ void env_dialog(GtkWidget* www, gpointer d)
  GtkWidget* dialog, *label[4], *button[3], *entry[4], *table;
  char **t_entry;
  int i;
+ debug("%s:%d",__FILE__,__LINE__);
  t_entry = (char**)malloc(4<<2);
  for (i=0;i<4;i++) t_entry[i] = (char*)malloc(STDSTRING+1);
  for (i=0;i<4;i++)strcpy(t_entry[i], "");
@@ -729,8 +826,9 @@ void env_dialog(GtkWidget* www, gpointer d)
 void check_exec_proc(GtkWidget* txt, gpointer dlg)
 {
  int l;
+ debug("%s:%d",__FILE__,__LINE__);
  l = gtk_text_get_length(GTK_TEXT(txt));
- if (GTK_TEXT_INDEX(GTK_TEXT(txt), l-1)==';') 
+ if (GTK_TEXT_INDEX(GTK_TEXT(txt), l-1)==';')
  {
   gtk_text_set_point(GTK_TEXT(txt), l);
   gtk_text_backward_delete(GTK_TEXT(txt), 1);
@@ -742,6 +840,7 @@ void check_exec_proc(GtkWidget* txt, gpointer dlg)
 void query_executor(GtkWidget* w, gpointer data)
 {
  GtkWidget* window, *table, *text, *hscrollbar, *vscrollbar, *button[3];
+ debug("%s:%d",__FILE__,__LINE__);
  if (!connected)
    {
     gtk_dialog_printf("Connect to DB first!");
@@ -796,28 +895,6 @@ void query_executor(GtkWidget* w, gpointer data)
  gtk_main();
 }
 
-/* This is the GtkItemFactoryEntry structure used to generate new menus.
-   Item 1: The menu path. The letter after the underscore indicates an
-           accelerator key once the menu is open.
-   Item 2: The accelerator key for the entry
-   Item 3: The callback function.
-   Item 4: The callback action.  This changes the parameters with
-           which the function is called.  The default is 0.
-   Item 5: The item type, used to define what kind of an item it is.
-           Here are the possible values:
-           NULL               -> "<Item>"
-           ""                 -> "<Item>"
-           "<Title>"          -> create a title item
-           "<Item>"           -> create a simple item
-           "<CheckItem>"      -> create a check item
-           "<ToggleItem>"     -> create a toggle item
-           "<RadioItem>"      -> create a radio item
-           <path>             -> path of a radio item to link against
-           "<Separator>"      -> create a separator
-           "<Branch>"         -> create an item to hold sub items (optional)
-           "<LastBranch>"     -> create a right justified branch
-*/			/* FIXME */			/* ADD TOOLBAR WITH SHORTCUTS TO MENU */
-			/* FIXME */			/* TESTS, TOOLBARS, RELATIONS ... */
 static GtkItemFactoryEntry menu_items[] = {
   { "/_File",         		NULL,         NULL, 0, "<Branch>" },
   { "/File/_New DB",     	"<control>N", create_database, 0, NULL },	/* DONE */
@@ -825,6 +902,7 @@ static GtkItemFactoryEntry menu_items[] = {
   { "/File/_Create DB from File","<control>C", create_DB_from_file, 0, NULL },  /* DONE */
   { "/File/_Save DB to File",	"<control>S", save_database, 0, NULL },		/* DONE */
   { "/File/_Drop DB",	"<control>D", drop_database, 0, NULL },			/* DONE */
+  { "/File/_Copy DB",	"<control>P", copy_database, 0, NULL },	
   { "/File/_Execute SQL Query",	"<control>E", query_executor, 0, NULL },	/* DONE */
   { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
   { "/File/Quit",     "<control>Q", (void (*)(GtkWidget*,gpointer))menu_quit, 0, NULL }, /* DONE */
@@ -832,14 +910,15 @@ static GtkItemFactoryEntry menu_items[] = {
   { "/Options/Test",  "<control>T",         exec_test_prog, 0, NULL },		/* DONE */
   { "/Options/Environment",     NULL,       env_dialog, 0, NULL },		/* DONE */
   { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
-  { "/_Help/About",   NULL,         about_box, 0, NULL },			/* ADD PIXMAPS ETC. */
+  { "/_Help/About",   NULL,         about_box, 0, NULL },			/* DONE */
 };
 
-void get_main_menu( GtkWidget  *window,GtkWidget **menubar )
+void get_main_menu(GtkWidget *window, GtkWidget **menubar)
 {
   GtkItemFactory *item_factory;
   GtkAccelGroup *accel_group;
   gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
+ debug("%s:%d",__FILE__,__LINE__);
   accel_group = gtk_accel_group_new ();
   item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",accel_group);
   gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
@@ -851,6 +930,7 @@ void get_main_menu( GtkWidget  *window,GtkWidget **menubar )
 int dummy_connect(int cn, char* templ)
 {
  char errstr[MAX_ENTRY_LENGTH];
+ debug("%s:%d",__FILE__,__LINE__);
  if (cn==1)
   {
    connection = gtk_connect_db(templ, errstr);
@@ -867,50 +947,23 @@ int dummy_connect(int cn, char* templ)
 }
 
 
-void todo()
-{
- printf("przejrzec dokladnie przyklady w gtkex/\n");
- printf("see entire DB, zakladki (notepads)i w kazdej tablica, nazwy kolumn\n");
- printf("nazwy kolumn w wyniku zapytania SQL, jezeli #rows>0\n");
- printf("dodac to do klasy funkcji: execute_(...)\n");
- printf("toolbar ze skrotami, oddzielny plik: toolbar.[ch]\n");
- printf("header dla funkcji toolbara: main.h (naglowki funkcji z main menu)\n");
- printf("tablice in tree-view, typy danych w combo\n");
- printf("dla tabeli: tablica: nazwa kolumny, typ (combo)\n");
- printf("operacje na tablicach itp. w menu kontekstowym\n");
- printf("jakis obrazek w about_box, niech uzyje mjpeg.c, mjpeg.h\n");
- printf("aby uzyc mjpeg, trzeba gtk_window_get_xwindow, i gdk events\n");
- printf("trigery: w menu kontekstowym, i add/delete table etc.\n");
- printf("kreator tworzenia tablicy, zapytania (Create Query? do menu)\n");
- printf("Potem zapisanie kwerendy i wykonanie jej (w kreatorze\n");
- printf("zapytania musi byc opcja wczytania z pliku juz gotowego)\n");
- printf("dodawanie tabeli, usuwanie tabeli (modyf. w combo i alter_tab)\n");
- printf("dodawanie danych do tablicy, jako caly wiersz i <Add>\n");
- printf("kasowanie calych wierszy w tablicy\n");
- printf("modyfikacja wierszy w tablicy <SIC!>\n");
- printf("synchronizacja, jak wykonanie kwerendy w Qexec to update ctrls\n");
- printf("kreator tworzenia triggera, zapis do pliku, wczytanie trg\n");
- printf("WIDOKU RELACYJNEGO NAJPRAWDOPODOBNIEJ NIE BEDZIE\n");
- printf("CHYBA ZE SIE DOWIEM JAK OKRESLIC CZY DANA JEST KUCZEM pk LUB fk\n");
- printf("! kreator tworzenia triggera w C, kompilowanie do *.so\n");
- printf("kreator tworzenia widoku, zapis kodu SQL widoku i odczyt\n");
-}
-
-
 int parse_cmdline(int lb, char** par, GtkWidget* w)
 {
  char* dbname;
  char errstr[MAX_ENTRY_LENGTH];
  int gotdb,ch;
+ int tmp;
+ debug("%s:%d",__FILE__,__LINE__);
  gotdb=0;
  dbname = NULL;
- while ((ch = getopt(lb,par,"d:s:p:b:he")) != -1)
+ while ((ch = getopt(lb,par,"d:s:p:b:t:he")) != -1)
    {
     switch (ch)
       {
        case 's': setenv("PGHOST", optarg, 1); break;
        case 'p': setenv("PGPORT", optarg, 1); break;
        case 'b': setenv("PGDATA", optarg, 1); break;
+       case 't': tmp=atoi(optarg); if (tmp<=0) fatal("timeout value: %d too small", tmp); timeout=tmp; break;
        case 'd': gotdb=1; dbname = malloc(strlen(optarg)+1); strcpy(dbname, optarg); break;
        case 'e': gotdb=1; env_dialog(w,(gpointer)1); return 0;
        case 'h':
@@ -919,9 +972,10 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
 				 "-h this help\n"
 				 "-s server name\n"
 				 "-p port\n"
-				 "-b base dir"
+				 "-b base dir\n"
+				 "-t timeout connection"
 				 ); return 1;
-       default: 
+       default:
 		gtk_dialog_printf("Unrecognized command line option\noptions are:\n-d data_base_name\n"
 				   "-h help"); return 1;
       }
@@ -966,7 +1020,7 @@ int main(int lb, char **par)
   GtkWidget* toolbar;
   char winname[MAX_ENTRY_LENGTH];
   int err;
-  todo();
+  debug("%s:%d",__FILE__,__LINE__);
   gtk_init (&lb, &par);
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   /*gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (gtk_main_quit), "WM destroy");*/
