@@ -6,12 +6,13 @@
  written in VIM editor, ctags used, CVS used
  Currently only one author: MOrgoth DBMA
  FILE: pgxexplorer.h */
-#define STDSTRING 100
-#define MAXPATH 1024
+#define STDSTRING 1024
+#define MAXPATH 2048
 #define TEMPLATE "template1"
 PGconn* connection;	/* variables here */
 char* currentbase;
-char tmpstr[1024];
+char tmpstr[STDSTRING];
 GtkWidget* main_wnd;
 GtkWidget* query_exec;
 int connected=0;
+int timeout=10;
