@@ -11,22 +11,67 @@
 #include "query.h"
 #include "signals.h"
 /*FIXME dodac obsluge debug, error ==> DlgWindow*/
+/* THERE WOULDN'T BE ANY DEBUG LOG TO FILE ETC. */
+
+int ping_conn;
+extern int timeout;
+
+int connAlive(PGconn* c)
+{
+ PGresult* r;
+ int stat;
+ debug("%s:%d", __FILE__,__LINE__);
+ ping_conn=1;
+ alarm_clock(timeout);
+ r= PQexec(c, "SELECT datname FROM pg_database");
+ alarm_stop();
+ if (!r)
+   {
+    debug("PQresult is NULL %s\n", PQerrorMessage(c));
+    return 0;
+   }
+ stat = PQresultStatus(r);
+ if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
+   {
+    debug("PQexec error: %s %s",PQcmdStatus(r),PQresultErrorMessage(r));
+    return 0;
+   }
+ return ping_conn;
+}
+
+void disconnect_db(PGconn** c)
+{
+ debug("%s:%d", __FILE__,__LINE__);
+ if (*c) PQfinish(*c);
+ else warn("connect is null: %s:%d", __FILE__,__LINE__);
+ *c = NULL;
+}
 
 PGconn* connect_db(char* arg)
 {
  PGconn* conn;
  char* strarg;
  conn=0;
- strarg = (char*)malloc(strlen(arg)+16);
+ debug("%s:%d", __FILE__,__LINE__);
+ strarg = (char*)malloc(strlen(arg)+32);
  debug("\nconnect to database %s\n", arg);
- sprintf(strarg,"dbname=%s", arg);
- alarm_clock(10);
+ sprintf(strarg,"dbname=template1");
+ alarm_clock(timeout);
  conn = PQconnectdb(strarg);
  alarm_stop();
+ if (!conn) { error("cannot connect to template1 DB"); free(strarg); return NULL; }
+ if (PQstatus(conn)==CONNECTION_BAD) {  error("cannot connect to template1 DB"); free(strarg); return NULL; }
+ disconnect_db(&conn);
+ sprintf(strarg,"dbname=%s", arg);
+ alarm_clock(timeout);
+ conn = PQconnectdb(strarg);
+ alarm_stop();
+ if (!conn) { error("connection is NULL %s:%d", __FILE__,__LINE__); return NULL; }
  if (PQstatus(conn) == CONNECTION_BAD)
    {
     error("\nCannot connect to DB:%s\n", arg);
     ERR
+    free(strarg);
     return NULL;
    }
  free(strarg);
@@ -39,11 +84,20 @@ PGconn* gtk_connect_db(char* arg, char* errstr)
  PGconn* conn;
  char* strarg;
  conn=0;
+ debug("%s:%d", __FILE__,__LINE__);
  strarg = (char*)malloc(strlen(arg)+16);
- sprintf(strarg,"dbname=%s", arg);
- alarm_clock(10);
+ sprintf(strarg,"dbname=template1");
+ alarm_clock(timeout);
  conn = PQconnectdb(strarg);
  alarm_stop();
+ if (!conn) { sprintf(errstr,"cannot connect to template1 DB"); free(strarg); return NULL; }
+ if (PQstatus(conn)==CONNECTION_BAD) {  sprintf(errstr,"cannot connect to template1 DB"); free(strarg); return NULL; }
+ disconnect_db(&conn);
+ sprintf(strarg,"dbname=%s", arg);
+ alarm_clock(timeout);
+ conn = PQconnectdb(strarg);
+ alarm_stop();
+ if (!conn) { error("connection is NULL %s:%d", __FILE__,__LINE__); free(strarg); return NULL; }
  if (PQstatus(conn) == CONNECTION_BAD)
   {
    sprintf(errstr, "Connection string:\ndbname=%s failed.", arg);
@@ -57,31 +111,27 @@ PGconn* gtk_connect_db(char* arg, char* errstr)
 
 void clear_result(PGresult** r)
 {
+ debug("%s:%d", __FILE__,__LINE__);
  if (*r) PQclear(*r);
- debug("clear result\n");
  *r=0;
 }
 
 
-void disconnect_db(PGconn** c)
-{
- debug("disconnect DB\n");
- if (*c) PQfinish(*c);
- *c = NULL;
-}
 
 
 PGresult* execute_silent_query(PGconn* c,  char* query)
 {
  PGresult* r;
  int stat;
+ debug("%s:%d", __FILE__,__LINE__);
  debug("SQLquery: \"%s\"\n", query);
+ if (!connAlive(c)) { debug("connection dead!\n"); return NULL; }
  if (!query)
    {
     debug("NULL query passed\n");
     return NULL;
    }
- if (!c) return NULL;
+ if (!c) { error("no connection"); return NULL; }
  r= PQexec(c, query);
  if (!r)
    {
@@ -99,25 +149,28 @@ PGresult* execute_silent_query(PGconn* c,  char* query)
 }
 
 
-char*** p3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, char* fmt,...)
+char*** p3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_meta, char* fmt,...)
 {
  va_list ap;
  char*** ret;
- char msg[MAX_QUERY_LENGTH];
+ char msg[MAX_QUERY_LENGTH*2];
+ debug("%s:%d", __FILE__,__LINE__);
+ if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
  vsprintf(msg,fmt,ap);
  va_end(ap);
- execute_printf(c, &ret, row, col, errstr, msg);
+ execute_printf(c, &ret, row, col, errstr, want_meta, msg);
  return ret;
 }
 
 
-void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, char* fmt,...)
+void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* fmt,...)
 {
  int stat;
  va_list ap;
- char query[MAX_QUERY_LENGTH];
+ char query[MAX_QUERY_LENGTH*2];
  PGresult* r;
+ debug("%s:%d", __FILE__,__LINE__);
  va_start(ap, fmt);
  vsprintf(query,fmt,ap);
  va_end(ap);
@@ -136,6 +189,7 @@ void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, c
     strcpy(errstr, "NULL query passed to DB");
     return;
    }
+ if (!connAlive(c)) { sprintf(errstr,"connection dead!\n"); return ; }
  r= PQexec(c, query);
  if (!r)
    {
@@ -153,7 +207,7 @@ void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, c
     clear_result(&r);
     return;
    }
- *res = pgres2pc3(r, row, col);
+ *res = pgres2pc3(r, row, col, want_meta);
  clear_result(&r);
 }
 
@@ -162,7 +216,8 @@ void execute_printf_query(PGconn* c,PGresult** r, char* errstr, char* fmt,...)
 {
  int stat;
  va_list ap;
- char query[MAX_QUERY_LENGTH];
+ char query[MAX_QUERY_LENGTH*2];
+ debug("%s:%d", __FILE__,__LINE__);
  va_start(ap, fmt);
  vsprintf(query,fmt,ap);
  va_end(ap);
@@ -178,6 +233,7 @@ void execute_printf_query(PGconn* c,PGresult** r, char* errstr, char* fmt,...)
     strcpy(errstr, "NULL query passed to DB");
     return;
    }
+ if (!connAlive(c)) { sprintf(errstr,"connection dead!\n"); return ; }
  *r= PQexec(c, query);
  if (!*r)
    {
@@ -199,6 +255,7 @@ PGresult* execute_query(PGconn* c,  char* query)
 {
  PGresult* r;
  int stat;
+ debug("%s:%d", __FILE__,__LINE__);
  debug("SQLquery: \"%s\"\n", query);
  if (!query)
    {
@@ -206,6 +263,7 @@ PGresult* execute_query(PGconn* c,  char* query)
     return NULL;
    }
  if (!c) return NULL;
+ if (!connAlive(c)) { debug("connection dead!\n"); return NULL; }
  r= PQexec(c, query);
  if (!r)
    {
@@ -223,16 +281,20 @@ PGresult* execute_query(PGconn* c,  char* query)
 }
 
 
-char*** pgres2pc3(const PGresult* r, int* a, int* b)
+char*** pgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
 {
  char*** result;
+ char* tmp;
  int i,j;
+ debug("%s:%d", __FILE__,__LINE__);
+ debug("pgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
  i=j=0;
- debug("pgres2pc3: %p\n", (void*)r);
  *a = PQntuples(r);
  *b = PQnfields(r);
  /*printf("ntuples=%d,%d\n", *a,*b);*/
+ if (!want_meta)
+ {
  result = (char***)malloc((*a)<<2);
  if (!result) { ERR error("malloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
@@ -257,21 +319,59 @@ char*** pgres2pc3(const PGresult* r, int* a, int* b)
       }
     strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
    }
-#ifdef DEBUG
+/*#ifdef DEBUG
   debug("char*** result is (%d:%d):\n",i,j);
   for (i=0;i<*a;i++)
     {
      for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
      ln();
     }
-#endif
+#endif*/
  return result;
+ }
+ else 
+ {
+ (*a)++;
+ result = (char***)malloc((*a)<<2);
+ if (!result) { ERR error("malloc error\n"); return NULL; }
+ for (i=0;i<*a;i++)
+   {
+    result[i] = (char**)malloc((*b)<<2);
+    if (!result[i])
+      {
+       for (j=0;j<i;j++) free(result[i]);
+       ERR error("malloc_i error!\n");
+       return NULL;
+      }
+   }
+ for (i=0;i<*b;i++)
+   {
+    tmp = PQfname(r,i);
+    result[0][i] = (char*)malloc(strlen(tmp)+1);
+    if (!result[0][i]) { ERR error("malloc_namej error"); return NULL; }
+    strcpy(result[0][i], tmp);
+   }
+ for (i=1;i<*a;i++)
+ for (j=0;j<*b;j++)
+   {
+    result[i][j] = (char*)malloc(PQgetlength(r,i-1,j)+1);
+    if (!result[i][j])
+      {
+       free_p3c(&result,*a,*b);
+       ERR error("malloc_ij error\n");
+       return NULL;
+      }
+    strcpy(result[i][j], (char*)PQgetvalue(r,i-1,j));
+   }
+  return result;
+ }
 }
 
 
 void free_p3c(char**** p, int a, int b)
 {
  int i,j;
+ debug("%s:%d", __FILE__,__LINE__);
  debug("free_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
  for (i=0;i<a;i++) if (!(*p)[i]) return ;
