@@ -8,8 +8,8 @@
  FILE: query.h */
 #ifndef __MORGOTHDBMA_QUERY_H__
 #define __MORGOTHDBMA_QUERY_H__
-#define MAX_ERROR_STRING 0XFF
-#define MAX_QUERY_LENGTH 2048
+#define MAX_ERROR_STRING 4096
+#define MAX_QUERY_LENGTH 4096
 #include <libpq-fe.h>
 #include <string.h>
 #include "consoleout.h"
@@ -20,8 +20,8 @@ void disconnect_db(PGconn**);
 PGresult* execute_silent_query(PGconn*,char*);
 PGresult* execute_query(PGconn*,char*);
 void execute_printf_query(PGconn*,PGresult**,char*, char*,...);
-void execute_printf(PGconn*,char****, int*, int*, char*, char*,...);
-char*** p3c_execute_printf(PGconn*,int*, int*,char*,char*,...);
-char*** pgres2pc3(const PGresult*,int*,int*);
+void execute_printf(PGconn*,char****, int*, int*, char*, int, char*,...);
+char*** p3c_execute_printf(PGconn*,int*, int*,char*,int, char*,...);
+char*** pgres2pc3(const PGresult*,int*,int*, int);
 void free_p3c(char****,int,int);
 #endif
