@@ -8,20 +8,29 @@
  FILE: signals.c */
 #include "signals.h"
 
+extern int ping_conn;
+
 void priv_sig_proc()
 {
- gtk_dialog_printf("Tiomeout!\n");
+ debug("%s:%d", __FILE__,__LINE__);
+ ping_conn=0;
+ gtk_dialog_printf("Tiomeout!\nConnection lost!\nSystem will now exit");
+ exit(1);
+ ping_conn=0;
+ 
 }
 
 
 void alarm_clock(int sec)
 {
+ debug("%s:%d", __FILE__,__LINE__);
  alarm(sec);
 }
 
 
 void alarm_stop()
 {
+ debug("%s:%d", __FILE__,__LINE__);
  alarm(0);
 }
 
@@ -29,6 +38,8 @@ void alarm_stop()
 void setup_signals()
 {
  static struct sigaction act;
+ ping_conn=0;
+ debug("%s:%d", __FILE__,__LINE__);
  act.sa_handler = priv_sig_proc;
  sigfillset(&(act.sa_mask));
  sigfillset(&(act.sa_mask));
